/*
 *    GS3Browse.java
 *    Copyright (C) 2005 New Zealand Digital Library, http://www.nzdl.org
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.greenstone.gsdl3.service;

// Greenstone classes
import org.greenstone.gsdl3.util.GS3OID;
import org.greenstone.gsdl3.util.GSXML;
import org.greenstone.gsdl3.util.GSFile;
import org.greenstone.gsdl3.util.SQLQuery;

// XML classes
import org.w3c.dom.Document;
import org.w3c.dom.Element; 
import org.w3c.dom.NodeList;

// General Java classes
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.io.File;

/** Greenstone 3 collection classifier service
 *
 * @author <a href="mailto:kjdon@cs.waikato.ac.nz">Katherine Don</a>
 */
public class GS3Browse
    extends AbstractBrowse
{

    protected SQLQuery database = null;

    /** constructor */
    public GS3Browse()
    {
	this.database = new SQLQuery();

    }

    public void cleanUp() {
	super.cleanUp();
	this.database.closeConnection();
    }
    
    public boolean configure(Element info, Element extra_info)
    {
	System.err.println("Configuring GS3Browse...");
	
	// open the database for querying
	// the database name is a combination of site name and collection name

	// check that site_home is set
	if (this.site_home == null || this.site_home.equals("")) {
	    System.err.println("GS3Retrieve Error: site_home is not set, so cannot work out the site name and cannot determine the database name");
	    return false;
	}
	String site_name = this.site_home.substring(this.site_home.lastIndexOf(File.separator)+1);
	if (site_name.equals("")) {
	    System.err.println("GS3Retrieve Error: Cannot extract the site name from site home: "+this.site_home);
	    return false;
	}
	if (!database.setDatabase(site_name+"_"+this.cluster_name)) {
	    System.err.println("GS3Retrieve Error: Could not open SQL database!");
	    return false;
	}
	
	// generic configure
	return super.configure(info, extra_info);
    }

    /** if id ends in .fc, .pc etc, then translate it to the correct id */
    protected String translateId(String node_id) {
	// should really use a sql query
	return GS3OID.translateOID(node_id);
    }

        /** returns the document type of the doc that the specified node 
	belongs to. should be one of 
	GSXML.DOC_TYPE_SIMPLE, 
	GSXML.DOC_TYPE_PAGED, 
	GSXML.DOC_TYPE_HIERARCHY
    */
    protected String getDocType(String node_id) {
	boolean hierarchical = false;
	if (!GS3OID.isDocTop(node_id) || database.isHierarchicalDocument(node_id) ) {
	    hierarchical = true;
	} 
	// what about paged???
	if (!hierarchical) {
	    // a simple document
	    return GSXML.DOC_TYPE_SIMPLE;
	} else {
	    // a hierarchical doc
	    return GSXML.DOC_TYPE_HIERARCHY;
	}
	// don't have paged yet.
   }
    
    /** returns the id of the root node of the document containing node 
     * node_id. . may be the same as node_id */
    protected String getRootId(String node_id) {
	return GS3OID.getDocID(node_id);
    }
    
   /** returns a list of the child ids in order, null if no children */
    protected ArrayList getChildrenIds(String node_id) {
	ArrayList documents = database.getClassifierDocChildren(node_id);
	if (documents == null) {
	    documents = new ArrayList();
	}
	ArrayList children = database.getClassifierChildren(node_id);
	if (children != null) {
	    documents.addAll(children);
	}
	if (documents.size()==0) {
	    return null;
	}
	return documents;
    }

    /** returns the node id of the parent node, null if no parent */
    protected String getParentId(String node_id){
	String parent = GS3OID.getParent(node_id);
	if (parent.equals(node_id)) {
	    return null;
	}
	return parent;
    }
    
    /** get the metadata for the classifier node node_id
     * returns a metadataList element:
     * <metadataList><metadata name="xxx">value</metadata></metadataList>
     * if all_metadata is true, returns all available metadata, otherwise just
     * returns requested metadata
     */
    // assumes only one value per metadata
    protected Element getMetadataList(String node_id, boolean all_metadata, ArrayList metadata_names) {
	Element metadata_list = this.doc.createElement(GSXML.METADATA_ELEM+GSXML.LIST_MODIFIER);

	if (all_metadata) {
	    // TODO
	    

	} else {
	    for (int m = 0; m < metadata_names.size(); m++) {
		String metadata = (String) metadata_names.get(m);
		String value = null;
		value = database.getClassifierMetadata(node_id, metadata);
		if (value != null) {
		    System.out.println("found value "+value);
		    GSXML.addMetadata(this.doc, metadata_list, metadata, value);
		}
	    }
	}

	return metadata_list;
    }

    /** returns the structural information asked for.
     * info_type may be one of
     * INFO_NUM_SIBS, INFO_NUM_CHILDREN, INFO_SIB_POS
     */
    protected String getStructureInfo(String doc_id, String info_type) {
	return null;
    }

    /** returns true if the id refers to a document (rather than 
     * a classifier node)
     */
    protected boolean isDocumentId(String node_id){
	if (node_id.startsWith("CL")) {
	    return false;
	}
	return true;
    }

}
